from yell import notify, Notification
from yell.decorators import notification


try:
    import unittest2 as unittest
except ImportError:
    import unittest
    
@notification(name='decorator')
def decorator_notification0(*args, **kwargs):
    return 'Decorator 0', args, kwargs

@notification(name='decorator')
def decorator_notification1(*args, **kwargs):
    return 'Decorator 1', args, kwargs

class ClassNotification0(Notification):
    name = 'class'
    
    def notify(self, *args, **kwargs):
        return 'Class 0', args, kwargs

class ClassNotification1(Notification):
    name = 'class'
    
    def notify(self, *args, **kwargs):
        return 'Class 1', args, kwargs

class AssertMixin(object):
    retval = None
    def _assert_results(self, results):
        for i, result in enumerate(results):
            self.assertTrue('%s %s' % (self.retval, i) == result[0], """ Return value '%s %s' not equal to '%s' """ % (self.retval, i, result[0]))

            self.assertTrue('Arg1' == result[1][0], """ Expected value 'Arg1' does not match received value '%s' """ % result[1][0])
            self.assertTrue('Arg2' == result[1][1], """ Expected value 'Arg2' does not match received value '%s' """ % result[1][1])
            
            self.assertTrue('Kwarg1' in result[2].values())
            self.assertTrue('Kwarg2' in result[2].values())
            

class TestDecoratorNotification(AssertMixin, unittest.TestCase):
    retval = 'Decorator'

    def test_notifying_with_decorator(self):
        results = notify('decorator', 'Arg1', 'Arg2', kwarg1='Kwarg1', kwarg2='Kwarg2')
        self.assertEqual(2, len(results))           
        self._assert_results(results)

        results = decorator_notification0.notify('Arg1', 'Arg2', kwarg1='Kwarg1', kwarg2='Kwarg2')
        self._assert_results(results)
        self.assertEqual(2, len(results))           
    

    def test_notifying_once_with_decorator(self):
        results = decorator_notification0.notify_once('Arg1', 'Arg2', kwarg1='Kwarg1', kwarg2='Kwarg2')
        self.assertEqual(1, len(results))
        self._assert_results(results)


class TestClassNotification(AssertMixin, unittest.TestCase):
    retval = 'Class'
    
    def test_notifying_with_class(self):
        results = notify('class', 'Arg1', 'Arg2', kwarg1='Kwarg1', kwarg2='Kwarg2')
        self.assertEqual(2, len(results))           
        self._assert_results(results)

    def test_notifying_once_with_class(self):
        results = notify('class', 'Arg1', 'Arg2', kwarg1='Kwarg1', kwarg2='Kwarg2', backends=[ClassNotification0])
        self.assertEqual(1, len(results))
        self._assert_results(results)

    def test_notifying_with_send_static_method(self):
        results = ClassNotification0.send('Arg1', 'Arg2', kwarg1='Kwarg1', kwarg2='Kwarg2')
        self.assertEqual(1, len(results))
        self._assert_results(results)



from __future__ import absolute_import
import os
from django.conf import settings
from django import template
from django.core.mail import send_mail, EmailMultiAlternatives
from yell import Notification



class EmailBackend(Notification):
    """
    Send emails via :attr:`django.core.mail.send_mail`
    """

    subject = None
    """ Email subject """
    body = None
    """ Email body """
    

    def get_subject(self, *args, **kwargs):
        """
        Return a subject. Override if you need templating or such. 
        """
        return self.subject
    
    def get_body(self, *args, **kwargs):
        """
        Return a body. Override if you need templating or such.
        """
        return self.body
    
    def get_from(self, *args, **kwargs):
        return settings.DEFAULT_FROM_EMAIL

    def get_to(self, *args, **kwargs):
        return kwargs.get('to')
    
    def notify(self, *args, **kwargs):
        
        return send_mail(
            self.get_subject(*args, **kwargs),
            self.get_body(*args, **kwargs),
            self.get_from(*args, **kwargs),
            self.get_to(*args, **kwargs),
        )

class MultipartEmailBackend(EmailBackend):
    """
    Send emails via :class:`django.core.mail.EmailMultiAlternatives`
    """
    default_content_type = 'text/plain'
    """
    The default content type. 
    """
    
    body = {}
    """
    Email body mapping content type to message. Requires at least a key for 
    :attr:`default_content_type`.
    """

    
    def get_body(self, *args, **kwargs):
        assert self.default_content_type in self.body, "No message body for default content type '%s'" % self.default_content_type
        return self.body
    
    def get_default_body(self, *args, **kwargs):
        return self.get_body(*args, **kwargs)[self.default_content_type]
    
    def notify(self, *args, **kwargs):        
        message = EmailMultiAlternatives(
            self.get_subject(*args, **kwargs),
            self.get_default_body(*args, **kwargs),
            self.get_from(*args, **kwargs),
            self.get_to(*args, **kwargs)
        )
        
        for content_type, body in self.get_body(*args, **kwargs).iteritems():
            if content_type == self.default_content_type: continue
            message.attach_alternative(body, content_type)
        
        return message.send()
        
class TemplatedEmailBackend(MultipartEmailBackend):
    """
    Generates email bodies from templates.
    
    :example:
    
    ::
    
        class SignupMessage(TemplatedEmailBackend):
            name = 'signup'
            subject = "Welcome to %s" % Site.objects.get_current().name
            content_types = ['text/plain', 'text/html']
        
    The `SignupMessage` class would look up following templates for the email body:
    
    * `yell/signup.txt`
    * `yell/signup.html`
    
    """
    content_types = (
        ('text/plain', '.txt'),
        ('text/html', '.html')
    )
    """
    Default content types to render
    """
        
    # Memoize
    _body = None
    
    def get_path(self, name, ext):
        """
        Get the path to a given template name. Override if you wish to
        store your templates in a custom folder outside of `yell/`.
        """
        return os.path.join('yell', '{0}{1}'.format(name, ext))

    def get_body(self, *args, **kwargs):
        """ 
        Render message bodies by using all the content types defined
        in :attr:`content_types`
        """
        if self._body is None:
            self._body = {}
            for ctype, ext in self.content_types:
                tpl = template.loader.get_template(self.get_path(self.name, ext))
                self._body[ctype] = tpl.render(template.Context(kwargs))
        return self._body
    



__version__ = "0.3.2"

import registry

class MetaNotification(type):
    """ 
    Metaclass that stores all notifications in the registry.
    """
    def __new__(cls, name, bases, attrs):
        Notification = super(MetaNotification, cls).__new__(cls, name, bases, attrs)

        if Notification.name is not None:
            registry.notifications[Notification.name] = registry.notifications.get(Notification.name, []) + [Notification]

            sendfn = lambda *args, **kwargs: notify(Notification.name, backends = [Notification], *args, **kwargs)
            setattr(Notification, 'send', staticmethod(sendfn))

        return Notification


class Notification(object):
    """
    Base class for any kind of notifications. Inherit from this class to create
    your own notification types and backends. 
    
    Subclasses need to implement :meth:`notify`.
    """
    __metaclass__ = MetaNotification

    name = None
    """
    A name for this notification.
    """
    
    def notify(self, *args, **kwargs):
        """
        A method that delivers a notification.
        """
        raise NotImplementedError

def notify(name, *args, **kwargs):
    """
    Send notifications. If ``backends==None``, all backends with the same name
    will be used to deliver a notification. 
    
    If ``backends`` is a list, only the specified backends will be used.
    
    :param name: The notification to send
    :param backends: A list of backends to be used or ``None`` to use all associated backends
    """
    assert name in registry.notifications, "'{0}' is not a valid notification.".format(repr(name))
    
    backends = kwargs.pop('backends', None)
    
    if backends is None:
        backends = registry.notifications[name]
    
    results = []

    for Backend in backends:
        backend = Backend()
        results.append(backend.notify(*args, **kwargs))
    
    return results





from __future__ import absolute_import
from celery.task import Task
from yell import Notification, notify, registry

class CeleryNotificationTask(Task):
    """ Dispatch and run the notification. """
    def run(self, name=None, backend=None, *args, **kwargs):
        """
        The Celery task. 
        
        Delivers the notification via all backends returned by :param:`backend`.
        """
        assert name is not None, "No 'name' specified to notify"
        assert backend is not None, "No 'backend' specified to notify with"
        
        backends = backend().get_backends(*args, **kwargs)
        notify(name, backends=backends, *args, **kwargs)
    
class CeleryNotification(Notification):
    """ 
    Delivers notifications through Celery. 
    
    :example:
    
    ::
    
        from yell import notify, Notification
    
        class EmailNotification(Notification):
            name = 'async'
            def notify(self, *args, **kwargs):
                # Deliver email 
                
        class DBNotification(Notification):
            name = 'async'
            def notify(self, *args, **kwargs):
                # Save to database
        
        class AsyncNotification(CeleryNotification):
            name = 'async'        
        
        notify('async', backends = [AsyncNotification],
            text = "This notification is routed through Celery before being sent and saved")
    
    In the above example when calling :attr:`yell.notify` will invoke ``EmailNotification`` and
    ``DBNotification`` once the task was delivered through Celery.
        
    """
    name = None
    """ 
    The name of this notification. Override in subclasses.
    """
    
    def get_backends(self, *args, **kwargs):
        """
        Return all backends the task should use to deliver notifications.
        By default all backends with the same :attr:`name` except for subclasses
        of :class:`CeleryNotifications` will be used.
        """
        return filter(lambda cls: not issubclass(cls, self.__class__), registry.notifications[self.name])

    
    def notify(self, *args, **kwargs):
        """ 
        Dispatches the notification to Celery
        """
        return CeleryNotificationTask.delay(name=self.name, backend=self.__class__, *args, **kwargs)





#!/usr/bin/env python
from setuptools import setup, find_packages

import yell

METADATA = dict(
    name='yell',
    version=yell.__version__,
    author='Alen Mujezinovic',
    author_email='flashingpumpkin@gmail.com',
    description='User notification library with pluggable backends. Compatible with popular frameworks such as Django, Flask, Celery.',
    long_description=open('README.rst').read(),
    url='https://github.com/caffeinehit/yell',
    keywords='django flask celery user notifications yell buffalo',
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: MacOS X',
        'Environment :: Web Environment',
        'Environment :: Other Environment',
        'Environment :: X11 Applications',
        'Framework :: Django',
        'Framework :: Paste',
        'Framework :: Pylons',
        'Framework :: TurboGears',
        'Framework :: Twisted',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Communications',
        'Topic :: Communications :: Email',
        'Topic :: Database',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    packages=find_packages(),
    test_suite='yell.tests',
)

if __name__ == '__main__':
    setup(**METADATA)



from yell import Notification, notify

class DecoratedNotification(Notification):
    func = None
    """
    The function that has been decorated. 
    """

    def notify(self, *args, **kwargs):
        return self.func(*args, **kwargs)

def notification(name=None, backends=None):
    """
    Decorator to simplify creation of notification backends. 
    
    :example:
    
    ::
    
        from yell.decorators import notification
        from yell import notify 
    
        @notification('like')
        def like_email(user, obj):
            send_mail("%s liked %s" % (user, obj), "No/Text", "noreply@example.com",
                [obj.user.email])
        
        @notification('like')
        def like_db(user, obj):
            DBNotification.objects.create(user = user, obj = obj, type = 'like')
    
        @notification('like')
        def like(*args, **kwargs):
            pass
        
        # Use all backends
        notify('like', user = user, obj = obj)        
        like.notify(user = user, obj = obj)
        
        # Only use the email backend
        like_email.notify_once(user = user, obj = obj)
        
    """

    def wrapper(func):
        def funcwrapper(self, *args, **kwargs):
            """ Wrapping the notification function so it doesn't receive `self` """
            return func(*args, **kwargs)
        
        NotificationCls = type('%sNotification' % name.lower().title(), (DecoratedNotification,), {
            'func': funcwrapper,
            'name': name
        })

        def notify_all(*args, **kwargs):
            """
            Sends this notification off to every backend with the configured name. 
            """
            return notify(name, *args, **kwargs)
        
        def notify_once(*args, **kwargs):
            """
            Sends this notification off only to the current backend.
            """
            return notify(name, backends=[NotificationCls], *args, **kwargs)
        
        func.notify = notify_all
        func.notify_once = notify_once
        
        return func
        
    return wrapper
